@@ -7,7 +7,8 @@
     extras <- match.call(expand.dots = FALSE)$...
     if (length(extras) > 0) 
     {
-        glsa <- names(as.list(args(multdrc)))
+#        glsa <- names(as.list(args(multdrc)))
+        glsa <- names(as.list(args(drm)))
         names(extras) <- glsa[pmatch(names(extras), glsa[-length(glsa)])]
         existing <- !is.na(match(names(extras), names(call)))
         for (a in names(extras)[existing]) call[[a]] <- extras[[a]]
